@@ -12,7 +12,7 @@ use swc_common::{FileName, Mark, Span, DUMMY_SP};
 use swc_ecma_ast::*;
 use swc_ecma_loader::resolve::Resolve;
 use swc_ecma_utils::{quote_ident, ExprFactory};
-use tracing::{debug, warn, Level};
+use tracing::{debug, trace, warn, Level};
 
 pub(crate) enum Resolver {
     Real {
@@ -100,15 +100,51 @@ where
     R: Resolve,
 {
     fn resolve_import(&self, base: &FileName, module_specifier: &str) -> Result<JsWord, Error> {
-        fn to_specifier(target_path: &str, is_file: Option<bool>, had_speicifer: bool) -> JsWord {
+        fn to_specifier(
+            target_path: &str,
+            is_file: Option<bool>,
+            orig_ext: Option<&str>,
+        ) -> JsWord {
             let mut p = PathBuf::from(target_path);
-            if !had_speicifer && is_file.unwrap_or_else(|| p.is_file()) {
-                if let Some(v) = p.extension() {
-                    if v == "ts" || v == "tsx" || v == "js" || v == "jsx" {
-                        p.set_extension("");
+
+            if cfg!(debug_assertions) {
+                trace!("to_specifier: orig_ext={:?}", orig_ext);
+            }
+
+            let dot_count = p
+                .file_name()
+                .map(|s| s.to_string_lossy())
+                .map(|v| v.as_bytes().iter().filter(|&&c| c == b'.').count())
+                .unwrap_or(0);
+
+            match orig_ext {
+                Some(orig_ext) => {
+                    if is_file.unwrap_or_else(|| p.is_file()) {
+                        if let Some(..) = p.extension() {
+                            if orig_ext == "ts"
+                                || orig_ext == "tsx"
+                                || orig_ext == "js"
+                                || orig_ext == "jsx"
+                                || dot_count == 1
+                            {
+                                p.set_extension(orig_ext);
+                            } else {
+                                p.set_extension("");
+                            }
+                        }
+                    }
+                }
+                _ => {
+                    if is_file.unwrap_or_else(|| p.is_file()) {
+                        if let Some(v) = p.extension() {
+                            if v == "ts" || v == "tsx" || v == "js" || v == "jsx" {
+                                p.set_extension("");
+                            }
+                        }
                     }
                 }
             }
+
             p.display().to_string().into()
         }
 
@@ -130,11 +166,13 @@ where
             debug!("invoking resolver");
         }
 
-        let had_speicifer = module_specifier
-            .split('/')
-            .last()
-            .map(|s| s.contains('.'))
-            .unwrap_or(false);
+        let orig_ext = module_specifier.split('/').last().and_then(|s| {
+            if s.contains('.') {
+                s.split('.').last()
+            } else {
+                None
+            }
+        });
 
         let target = self.resolver.resolve(base, module_specifier);
         let target = match target {
@@ -147,7 +185,7 @@ where
 
         let target = match target {
             FileName::Real(v) => v,
-            FileName::Custom(s) => return Ok(to_specifier(&s, None, had_speicifer)),
+            FileName::Custom(s) => return Ok(to_specifier(&s, None, orig_ext)),
             _ => {
                 unreachable!(
                     "Node path provider does not support using `{:?}` as a target file name",
@@ -188,7 +226,7 @@ where
                 return Ok(to_specifier(
                     &target.display().to_string(),
                     Some(is_file),
-                    had_speicifer,
+                    orig_ext,
                 ))
             }
         };
@@ -227,13 +265,9 @@ where
             Cow::Owned(format!("./{}", s))
         };
         if cfg!(target_os = "windows") {
-            Ok(to_specifier(
-                &s.replace('\\', "/"),
-                Some(is_file),
-                had_speicifer,
-            ))
+            Ok(to_specifier(&s.replace('\\', "/"), Some(is_file), orig_ext))
         } else {
-            Ok(to_specifier(&s, Some(is_file), had_speicifer))
+            Ok(to_specifier(&s, Some(is_file), orig_ext))
         }
     }
 }
